@@ -53,8 +53,9 @@ extern "C" {
 
 #define MAX_SECRET_LENGTH	(6 * 16) /* MUST be multiple of 16 */
 
-#define VENDOR(x)		(((x) >> 16) & 0xffff)
-#define ATTRID(x)		((x) & 0xffff)
+#define VENDOR_BIT_SIZE		32
+#define VENDOR(x)		(((x) >> VENDOR_BIT_SIZE) & 0xffffffff)
+#define ATTRID(x)		((x) & 0xffffffff)
 
 #define PW_MAX_MSG_SIZE		4096
 
@@ -111,7 +112,8 @@ typedef enum rc_attr_type {
 	PW_TYPE_IPADDR=2,	//!< The attribute is an IPv4 address in host-byte order.
 	PW_TYPE_DATE=3,		//!< The attribute contains a 32-bit number indicating the seconds since epoch.
 	PW_TYPE_IPV6ADDR=4,	//!< The attribute is an 128-bit IPv6 address.
-	PW_TYPE_IPV6PREFIX=5    //!< The attribute is an IPv6 prefix; the lvalue will indicate its size.
+	PW_TYPE_IPV6PREFIX=5,   //!< The attribute is an IPv6 prefix; the lvalue will indicate its size.
+	PW_TYPE_MAX=6   	//!< Maximum number of types (last+1)
 } rc_attr_type;
 
 /** \enum rc_standard_codes Standard RADIUS request codes
@@ -425,7 +427,7 @@ enum rc_vendor_attr_roaringpenguin {
 typedef struct dict_attr
 {
 	char              name[NAME_LENGTH + 1];	//!< attribute name.
-	int               value;			//!< attribute index.
+	uint64_t          value;			//!< attribute index and vendor number; use VENDOR() and ATTRID() to separate.
 	rc_attr_type      type;				//!< string, int, etc..
 	struct dict_attr *next;
 } DICT_ATTR;
@@ -434,14 +436,14 @@ typedef struct dict_value
 {
 	char               attrname[NAME_LENGTH +1];
 	char               name[NAME_LENGTH + 1];
-	int                value;
+	uint32_t           value;
 	struct dict_value *next;
 } DICT_VALUE;
 
 typedef struct dict_vendor
 {
 	char               vendorname[NAME_LENGTH +1];
-	int                vendorpec;
+	uint32_t           vendorpec;
 	struct dict_vendor *next;
 } DICT_VENDOR;
 
@@ -469,7 +471,7 @@ typedef enum rc_send_status {
 typedef struct rc_value_pair
 {
 	char               name[NAME_LENGTH + 1];	//!< attribute name if known.
-	unsigned           attribute;			//!< attribute numeric value of type rc_attr_id.
+	uint64_t           attribute;			//!< attribute numeric value of type rc_attr_id including vendor; use VENDOR() and ATTRID() to separate.
 	rc_attr_type	   type;			//!< attribute type.
 	uint32_t           lvalue;			//!< attribute value if type is PW_TYPE_INTEGER, PW_TYPE_DATE or PW_TYPE_IPADDR.
 	char               strvalue[AUTH_STRING_LEN + 1]; //!< contains attribute value in other cases.
@@ -560,13 +562,13 @@ typedef struct rc_aaa_ctx_st RC_AAA_CTX;
 
 /* avpair.c */
 
-VALUE_PAIR *rc_avpair_add (rc_handle const *rh, VALUE_PAIR **list, int attrid, void const *pval, int len, int vendorpec);
+VALUE_PAIR *rc_avpair_add (rc_handle const *rh, VALUE_PAIR **list, uint32_t attrid, void const *pval, int len, uint32_t vendorspec);
 int rc_avpair_assign (VALUE_PAIR *vp, void const *pval, int len);
-VALUE_PAIR *rc_avpair_new (rc_handle const *rh, int attrid, void const *pval, int len, int vendorpec);
+VALUE_PAIR *rc_avpair_new (rc_handle const *rh, uint32_t attrid, void const *pval, int len, uint32_t vendorspec);
 VALUE_PAIR *rc_avpair_gen(rc_handle const *rh, VALUE_PAIR *pair, unsigned char const *ptr,
-			  int length, int vendorpec);
-void rc_avpair_remove (VALUE_PAIR **list, int attrid, int vendorpec);
-VALUE_PAIR *rc_avpair_get (VALUE_PAIR *vp, int attrid, int vendorpec);
+			  int length, uint32_t vendorspec);
+void rc_avpair_remove (VALUE_PAIR **list, uint32_t attrid, uint32_t vendorspec);
+VALUE_PAIR *rc_avpair_get (VALUE_PAIR *vp, uint32_t attrid, uint32_t vendorspec);
 VALUE_PAIR *rc_avpair_copy(VALUE_PAIR *p);
 void rc_avpair_insert(VALUE_PAIR **a, VALUE_PAIR *p, VALUE_PAIR *b);
 void rc_avpair_free (VALUE_PAIR *pair);
@@ -624,11 +626,16 @@ rc_socket_type rc_get_socket_type(rc_handle * rh);
 
 int rc_read_dictionary (rc_handle *rh, char const *filename);
 int rc_read_dictionary_from_buffer (rc_handle *rh, char const *buf, size_t size);
-DICT_ATTR *rc_dict_getattr(rc_handle const *rh, int attribute);
+
+DICT_ATTR *rc_dict_addattr(rc_handle *rh, char const * namestr, uint32_t value, int type, uint32_t vendorspec);
+DICT_VALUE *rc_dict_addval(rc_handle *rh, char const * attrstr, char const * namestr, uint32_t value);
+DICT_VENDOR *rc_dict_addvend(rc_handle *rh, char const * vendorname, uint32_t value);
+
+DICT_ATTR *rc_dict_getattr(rc_handle const *rh, uint64_t attribute);
 DICT_ATTR *rc_dict_findattr(rc_handle const *rh, char const *attrname);
 DICT_VALUE *rc_dict_findval(rc_handle const *rh, char const *valname);
 DICT_VENDOR *rc_dict_findvend(rc_handle const *rh, char const *vendorname);
-DICT_VENDOR *rc_dict_getvend (rc_handle const *rh, int vendorpec);
+DICT_VENDOR *rc_dict_getvend (rc_handle const *rh, uint32_t vendorspec);
 DICT_VALUE *rc_dict_getval(rc_handle const *rh, uint32_t value, char const *attrname);
 void rc_dict_free(rc_handle *rh);
 
